import 'dart:io';

import 'package:me/utils/logger/log_utils.dart';
import 'package:path/path.dart';
import 'package:recase/recase.dart';

import '../../core/structure.dart';

enum TypeFile { binding, controller }

String? findFileFromNameAndType(
  String path,
  String name, {
  required TypeFile typeFile,
}) {
  path = Structure.replaceAsExpected(path: path);
  if (Directory(path).existsSync()) {
    var splitPath = Structure.safeSplitPath(path);
    splitPath
      ..remove('.')
      ..removeLast();

    var bindingPath = '';
    while (splitPath.isNotEmpty && bindingPath == '') {
      Directory(splitPath.join(separator))
          .listSync(recursive: true, followLinks: false)
          .forEach((element) {
        if (element is File) {
          var fileName = basename(element.path);
          LogService.error(_getFileByType(name: name, typeFile: typeFile));
          if (fileName == _getFileByType(name: name, typeFile: typeFile)) {
            bindingPath = element.path;
          }
        }
      });
      splitPath.removeLast();
    }
    return bindingPath;
  }
  return null;
}

String _getFileByType({
  required String name,
  required TypeFile typeFile,
}) {
  var fileType = typeFile == TypeFile.binding ? 'binding' : 'controller';
  return '${name.snakeCase}_$fileType.dart';
}
